from abc import abstractmethod

from rsocket.wellknown_mimetype import WellKnowMimeTypes, MIME_TYPES_BY_NAME


class CompositeMetadata:
    def __init__(self, source=None):
        if source is None:
            source = bytearray()
        self.source = source
        self.reader_index = 0

    def get_source(self):
        """composite metadata bytes for Payload"""
        return self.source

    def add_wellknown_metadata(self, mime_type_id, metadata_bytes):
        # mime id flag
        self.source.append(mime_type_id | 0x80)
        # metadata length
        self.source += len(metadata_bytes).to_bytes(3, byteorder='big')
        # metadata bytes
        self.source += metadata_bytes

    def add_custom_metadata(self, mime_type, metadata_bytes):
        if mime_type in MIME_TYPES_BY_NAME:
            self.add_wellknown_metadata(MIME_TYPES_BY_NAME[mime_type], metadata_bytes)
        else:
            mime_type_len = len(mime_type)
            # mime id/length
            self.source.append(mime_type_len)
            # mime type/content
            self.source += mime_type.encode("ascii")
            # metadata length
            self.source += mime_type_len.to_bytes(3, byteorder='big')
            # metadata bytes
            self.source += metadata_bytes

    def rewind(self):
        self.reader_index = 0

    def __iter__(self):
        return self

    def __next__(self):
        if len(self.source) <= self.reader_index:
            raise StopIteration
        else:
            mime_id = self.source[self.reader_index]
            # wellknown mime type
            if mime_id > 0x80:
                metadata_len_start = self.reader_index + 1
                metadata_len = int.from_bytes(self.source[metadata_len_start:metadata_len_start + 3], byteorder='big')
                metadata_bytes_start = metadata_len_start + 3
                metadata_bytes = self.source[metadata_bytes_start:metadata_bytes_start + metadata_len]
                self.reader_index = metadata_bytes_start + metadata_len
                return ReservedMimeTypeEntry(mime_id - 0x80, metadata_bytes)
            else:
                mime_type_start = self.reader_index + 1
                mime_type_len = mime_id
                mime_type = self.source[mime_type_start:mime_type_start + mime_type_len].decode("ascii")
                metadata_len_start = mime_type_start + mime_type_len
                metadata_len = int.from_bytes(self.source[metadata_len_start:metadata_len_start + 3], byteorder='big')
                metadata_bytes_start = metadata_len_start + 3
                metadata_bytes = self.source[metadata_bytes_start:metadata_bytes_start + metadata_len]
                self.reader_index = metadata_bytes_start + metadata_len
                return ExplicitMimeTimeEntry(mime_type, metadata_bytes)


class CompositeMetadataEntry:
    @abstractmethod
    def get_content(self):
        pass

    @abstractmethod
    def get_mime_type(self):
        pass


class ReservedMimeTypeEntry(CompositeMetadataEntry):
    def __init__(self, identifier, content):
        self.content = content
        self.mime_type = WellKnowMimeTypes.from_identifier(identifier).name

    def get_content(self):
        return self.content

    def get_mime_type(self):
        return self.mime_type


class ExplicitMimeTimeEntry(CompositeMetadataEntry):
    def __init__(self, mime_type, content):
        self.content = content
        self.mime_type = mime_type

    def get_mime_type(self):
        return self.mime_type

    def get_content(self):
        return self.content


class TaggingMetadata:

    def __init__(self, mime_type, source=None):
        self.mime_type = mime_type
        if source is None:
            source = bytearray()
        self.source = source
        self.source = source
        self.reader_index = 0

    def __iter__(self):
        return self

    def __next__(self):
        if len(self.source) <= self.reader_index:
            raise StopIteration
        else:
            tag_length = self.source[self.reader_index]
            tag_start = self.reader_index + 1
            tag = self.source[tag_start:tag_start + tag_length].decode("utf-8")
            self.reader_index = tag_start + tag_start + tag_length - 1
            return tag

    @classmethod
    def from_tags(cls, mime_type, tags):
        source = bytearray()
        for tag in tags:
            tag_bytes = bytes(tag, encoding='utf-8')
            source.append(len(tag_bytes) & 0x7F)
            source += tag_bytes

        return TaggingMetadata(mime_type, source)



"""
ReactiveStreams
~~~~~~~~~~~~~~~

Abstract base class definitions for ReactiveStreams Publisher/Subsciber.
"""

from reactivestreams.publisher import Publisher
from reactivestreams.subscriber import Subscriber
from reactivestreams.subscription import Subscription


def reactivestreams():
    return 'reactivestreams-0.1'



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: common_model.proto

from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='common_model.proto',
  package='com.elarian.hera.proto',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x12\x63ommon_model.proto\x12\x16\x63om.elarian.hera.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"B\n\x0c\x44\x61taMapValue\x12\x14\n\nstring_val\x18\x02 \x01(\tH\x00\x12\x13\n\tbytes_val\x18\x03 \x01(\x0cH\x00\x42\x07\n\x05value\"-\n\x04\x43\x61sh\x12\x15\n\rcurrency_code\x18\x01 \x01(\t\x12\x0e\n\x06\x61mount\x18\x02 \x01(\x01\"H\n\x0cIndexMapping\x12\x0b\n\x03key\x18\x01 \x01(\t\x12+\n\x05value\x18\x02 \x01(\x0b\x32\x1c.google.protobuf.StringValue\"v\n\rCustomerIndex\x12\x35\n\x07mapping\x18\x01 \x01(\x0b\x32$.com.elarian.hera.proto.IndexMapping\x12.\n\nexpires_at\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"\xa8\x01\n\x12\x41ppConnectionProps\x12\x15\n\rconnection_id\x18\x01 \x01(\t\x12\x33\n\x0f\x63onnection_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x30\n\nip_address\x18\x03 \x01(\x0b\x32\x1c.google.protobuf.StringValue\x12\x14\n\x0csimplex_mode\x18\x04 \x01(\x08\"\x93\x01\n\x0e\x43ustomerNumber\x12@\n\x08provider\x18\x01 \x01(\x0e\x32..com.elarian.hera.proto.CustomerNumberProvider\x12\x0e\n\x06number\x18\x02 \x01(\t\x12/\n\tpartition\x18\x03 \x01(\x0b\x32\x1c.google.protobuf.StringValue*\xfd\x01\n\x16\x43ustomerNumberProvider\x12(\n$CUSTOMER_NUMBER_PROVIDER_UNSPECIFIED\x10\x00\x12%\n!CUSTOMER_NUMBER_PROVIDER_FACEBOOK\x10\x01\x12%\n!CUSTOMER_NUMBER_PROVIDER_CELLULAR\x10\x02\x12%\n!CUSTOMER_NUMBER_PROVIDER_TELEGRAM\x10\x03\x12 \n\x1c\x43USTOMER_NUMBER_PROVIDER_WEB\x10\x04\x12\"\n\x1e\x43USTOMER_NUMBER_PROVIDER_EMAIL\x10\x05*\xc0\x02\n\x15\x43hannelNumberProvider\x12\'\n#CHANNEL_NUMBER_PROVIDER_UNSPECIFIED\x10\x00\x12\x1e\n\x1a\x43HANNEL_NUMBER_PROVIDER_AT\x10\x01\x12\"\n\x1e\x43HANNEL_NUMBER_PROVIDER_TWILIO\x10\x02\x12#\n\x1f\x43HANNEL_NUMBER_PROVIDER_MAILGUN\x10\x03\x12\"\n\x1e\x43HANNEL_NUMBER_PROVIDER_WEBAPP\x10\x04\x12$\n CHANNEL_NUMBER_PROVIDER_FACEBOOK\x10\x05\x12$\n CHANNEL_NUMBER_PROVIDER_TELEGRAM\x10\x06\x12%\n!CHANNEL_NUMBER_PROVIDER_SIMULATOR\x10\x07*\xc8\x01\n\tMediaType\x12\x1a\n\x16MEDIA_TYPE_UNSPECIFIED\x10\x00\x12\x14\n\x10MEDIA_TYPE_IMAGE\x10\x01\x12\x14\n\x10MEDIA_TYPE_AUDIO\x10\x02\x12\x14\n\x10MEDIA_TYPE_VIDEO\x10\x03\x12\x17\n\x13MEDIA_TYPE_DOCUMENT\x10\x04\x12\x14\n\x10MEDIA_TYPE_VOICE\x10\x05\x12\x16\n\x12MEDIA_TYPE_STICKER\x10\x06\x12\x16\n\x12MEDIA_TYPE_CONTACT\x10\x07*\x8f\x01\n\x16\x43ustomerEventDirection\x12(\n$CUSTOMER_EVENT_DIRECTION_UNSPECIFIED\x10\x00\x12$\n CUSTOMER_EVENT_DIRECTION_INBOUND\x10\x01\x12%\n!CUSTOMER_EVENT_DIRECTION_OUTBOUND\x10\x02*\x93\x01\n\x15\x43ustomerRequestOrigin\x12\'\n#CUSTOMER_REQUEST_ORIGIN_UNSPECIFIED\x10\x00\x12\'\n#CUSTOMER_REQUEST_ORIGIN_API_REQUEST\x10\x01\x12(\n$CUSTOMER_REQUEST_ORIGIN_CUSTOMER_TAG\x10\x02\x62\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_wrappers__pb2.DESCRIPTOR,google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,])

_CUSTOMERNUMBERPROVIDER = _descriptor.EnumDescriptor(
  name='CustomerNumberProvider',
  full_name='com.elarian.hera.proto.CustomerNumberProvider',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='CUSTOMER_NUMBER_PROVIDER_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CUSTOMER_NUMBER_PROVIDER_FACEBOOK', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CUSTOMER_NUMBER_PROVIDER_CELLULAR', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CUSTOMER_NUMBER_PROVIDER_TELEGRAM', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CUSTOMER_NUMBER_PROVIDER_WEB', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CUSTOMER_NUMBER_PROVIDER_EMAIL', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=742,
  serialized_end=995,
)
_sym_db.RegisterEnumDescriptor(_CUSTOMERNUMBERPROVIDER)

CustomerNumberProvider = enum_type_wrapper.EnumTypeWrapper(_CUSTOMERNUMBERPROVIDER)
_CHANNELNUMBERPROVIDER = _descriptor.EnumDescriptor(
  name='ChannelNumberProvider',
  full_name='com.elarian.hera.proto.ChannelNumberProvider',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='CHANNEL_NUMBER_PROVIDER_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CHANNEL_NUMBER_PROVIDER_AT', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CHANNEL_NUMBER_PROVIDER_TWILIO', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CHANNEL_NUMBER_PROVIDER_MAILGUN', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CHANNEL_NUMBER_PROVIDER_WEBAPP', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CHANNEL_NUMBER_PROVIDER_FACEBOOK', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CHANNEL_NUMBER_PROVIDER_TELEGRAM', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CHANNEL_NUMBER_PROVIDER_SIMULATOR', index=7, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=998,
  serialized_end=1318,
)
_sym_db.RegisterEnumDescriptor(_CHANNELNUMBERPROVIDER)

ChannelNumberProvider = enum_type_wrapper.EnumTypeWrapper(_CHANNELNUMBERPROVIDER)
_MEDIATYPE = _descriptor.EnumDescriptor(
  name='MediaType',
  full_name='com.elarian.hera.proto.MediaType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='MEDIA_TYPE_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MEDIA_TYPE_IMAGE', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MEDIA_TYPE_AUDIO', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MEDIA_TYPE_VIDEO', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MEDIA_TYPE_DOCUMENT', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MEDIA_TYPE_VOICE', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MEDIA_TYPE_STICKER', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MEDIA_TYPE_CONTACT', index=7, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1321,
  serialized_end=1521,
)
_sym_db.RegisterEnumDescriptor(_MEDIATYPE)

MediaType = enum_type_wrapper.EnumTypeWrapper(_MEDIATYPE)
_CUSTOMEREVENTDIRECTION = _descriptor.EnumDescriptor(
  name='CustomerEventDirection',
  full_name='com.elarian.hera.proto.CustomerEventDirection',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='CUSTOMER_EVENT_DIRECTION_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CUSTOMER_EVENT_DIRECTION_INBOUND', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CUSTOMER_EVENT_DIRECTION_OUTBOUND', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1524,
  serialized_end=1667,
)
_sym_db.RegisterEnumDescriptor(_CUSTOMEREVENTDIRECTION)

CustomerEventDirection = enum_type_wrapper.EnumTypeWrapper(_CUSTOMEREVENTDIRECTION)
_CUSTOMERREQUESTORIGIN = _descriptor.EnumDescriptor(
  name='CustomerRequestOrigin',
  full_name='com.elarian.hera.proto.CustomerRequestOrigin',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='CUSTOMER_REQUEST_ORIGIN_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CUSTOMER_REQUEST_ORIGIN_API_REQUEST', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CUSTOMER_REQUEST_ORIGIN_CUSTOMER_TAG', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1670,
  serialized_end=1817,
)
_sym_db.RegisterEnumDescriptor(_CUSTOMERREQUESTORIGIN)

CustomerRequestOrigin = enum_type_wrapper.EnumTypeWrapper(_CUSTOMERREQUESTORIGIN)
CUSTOMER_NUMBER_PROVIDER_UNSPECIFIED = 0
CUSTOMER_NUMBER_PROVIDER_FACEBOOK = 1
CUSTOMER_NUMBER_PROVIDER_CELLULAR = 2
CUSTOMER_NUMBER_PROVIDER_TELEGRAM = 3
CUSTOMER_NUMBER_PROVIDER_WEB = 4
CUSTOMER_NUMBER_PROVIDER_EMAIL = 5
CHANNEL_NUMBER_PROVIDER_UNSPECIFIED = 0
CHANNEL_NUMBER_PROVIDER_AT = 1
CHANNEL_NUMBER_PROVIDER_TWILIO = 2
CHANNEL_NUMBER_PROVIDER_MAILGUN = 3
CHANNEL_NUMBER_PROVIDER_WEBAPP = 4
CHANNEL_NUMBER_PROVIDER_FACEBOOK = 5
CHANNEL_NUMBER_PROVIDER_TELEGRAM = 6
CHANNEL_NUMBER_PROVIDER_SIMULATOR = 7
MEDIA_TYPE_UNSPECIFIED = 0
MEDIA_TYPE_IMAGE = 1
MEDIA_TYPE_AUDIO = 2
MEDIA_TYPE_VIDEO = 3
MEDIA_TYPE_DOCUMENT = 4
MEDIA_TYPE_VOICE = 5
MEDIA_TYPE_STICKER = 6
MEDIA_TYPE_CONTACT = 7
CUSTOMER_EVENT_DIRECTION_UNSPECIFIED = 0
CUSTOMER_EVENT_DIRECTION_INBOUND = 1
CUSTOMER_EVENT_DIRECTION_OUTBOUND = 2
CUSTOMER_REQUEST_ORIGIN_UNSPECIFIED = 0
CUSTOMER_REQUEST_ORIGIN_API_REQUEST = 1
CUSTOMER_REQUEST_ORIGIN_CUSTOMER_TAG = 2



_DATAMAPVALUE = _descriptor.Descriptor(
  name='DataMapValue',
  full_name='com.elarian.hera.proto.DataMapValue',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='string_val', full_name='com.elarian.hera.proto.DataMapValue.string_val', index=0,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bytes_val', full_name='com.elarian.hera.proto.DataMapValue.bytes_val', index=1,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='value', full_name='com.elarian.hera.proto.DataMapValue.value',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=111,
  serialized_end=177,
)


_CASH = _descriptor.Descriptor(
  name='Cash',
  full_name='com.elarian.hera.proto.Cash',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='currency_code', full_name='com.elarian.hera.proto.Cash.currency_code', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='amount', full_name='com.elarian.hera.proto.Cash.amount', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=179,
  serialized_end=224,
)


_INDEXMAPPING = _descriptor.Descriptor(
  name='IndexMapping',
  full_name='com.elarian.hera.proto.IndexMapping',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='com.elarian.hera.proto.IndexMapping.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='com.elarian.hera.proto.IndexMapping.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=226,
  serialized_end=298,
)


_CUSTOMERINDEX = _descriptor.Descriptor(
  name='CustomerIndex',
  full_name='com.elarian.hera.proto.CustomerIndex',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='mapping', full_name='com.elarian.hera.proto.CustomerIndex.mapping', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='expires_at', full_name='com.elarian.hera.proto.CustomerIndex.expires_at', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=300,
  serialized_end=418,
)


_APPCONNECTIONPROPS = _descriptor.Descriptor(
  name='AppConnectionProps',
  full_name='com.elarian.hera.proto.AppConnectionProps',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='connection_id', full_name='com.elarian.hera.proto.AppConnectionProps.connection_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='connection_time', full_name='com.elarian.hera.proto.AppConnectionProps.connection_time', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ip_address', full_name='com.elarian.hera.proto.AppConnectionProps.ip_address', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='simplex_mode', full_name='com.elarian.hera.proto.AppConnectionProps.simplex_mode', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=421,
  serialized_end=589,
)


_CUSTOMERNUMBER = _descriptor.Descriptor(
  name='CustomerNumber',
  full_name='com.elarian.hera.proto.CustomerNumber',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='provider', full_name='com.elarian.hera.proto.CustomerNumber.provider', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='number', full_name='com.elarian.hera.proto.CustomerNumber.number', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='partition', full_name='com.elarian.hera.proto.CustomerNumber.partition', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=592,
  serialized_end=739,
)

_DATAMAPVALUE.oneofs_by_name['value'].fields.append(
  _DATAMAPVALUE.fields_by_name['string_val'])
_DATAMAPVALUE.fields_by_name['string_val'].containing_oneof = _DATAMAPVALUE.oneofs_by_name['value']
_DATAMAPVALUE.oneofs_by_name['value'].fields.append(
  _DATAMAPVALUE.fields_by_name['bytes_val'])
_DATAMAPVALUE.fields_by_name['bytes_val'].containing_oneof = _DATAMAPVALUE.oneofs_by_name['value']
_INDEXMAPPING.fields_by_name['value'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_CUSTOMERINDEX.fields_by_name['mapping'].message_type = _INDEXMAPPING
_CUSTOMERINDEX.fields_by_name['expires_at'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_APPCONNECTIONPROPS.fields_by_name['connection_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_APPCONNECTIONPROPS.fields_by_name['ip_address'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_CUSTOMERNUMBER.fields_by_name['provider'].enum_type = _CUSTOMERNUMBERPROVIDER
_CUSTOMERNUMBER.fields_by_name['partition'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
DESCRIPTOR.message_types_by_name['DataMapValue'] = _DATAMAPVALUE
DESCRIPTOR.message_types_by_name['Cash'] = _CASH
DESCRIPTOR.message_types_by_name['IndexMapping'] = _INDEXMAPPING
DESCRIPTOR.message_types_by_name['CustomerIndex'] = _CUSTOMERINDEX
DESCRIPTOR.message_types_by_name['AppConnectionProps'] = _APPCONNECTIONPROPS
DESCRIPTOR.message_types_by_name['CustomerNumber'] = _CUSTOMERNUMBER
DESCRIPTOR.enum_types_by_name['CustomerNumberProvider'] = _CUSTOMERNUMBERPROVIDER
DESCRIPTOR.enum_types_by_name['ChannelNumberProvider'] = _CHANNELNUMBERPROVIDER
DESCRIPTOR.enum_types_by_name['MediaType'] = _MEDIATYPE
DESCRIPTOR.enum_types_by_name['CustomerEventDirection'] = _CUSTOMEREVENTDIRECTION
DESCRIPTOR.enum_types_by_name['CustomerRequestOrigin'] = _CUSTOMERREQUESTORIGIN
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DataMapValue = _reflection.GeneratedProtocolMessageType('DataMapValue', (_message.Message,), {
  'DESCRIPTOR' : _DATAMAPVALUE,
  '__module__' : 'common_model_pb2'
  # @@protoc_insertion_point(class_scope:com.elarian.hera.proto.DataMapValue)
  })
_sym_db.RegisterMessage(DataMapValue)

Cash = _reflection.GeneratedProtocolMessageType('Cash', (_message.Message,), {
  'DESCRIPTOR' : _CASH,
  '__module__' : 'common_model_pb2'
  # @@protoc_insertion_point(class_scope:com.elarian.hera.proto.Cash)
  })
_sym_db.RegisterMessage(Cash)

IndexMapping = _reflection.GeneratedProtocolMessageType('IndexMapping', (_message.Message,), {
  'DESCRIPTOR' : _INDEXMAPPING,
  '__module__' : 'common_model_pb2'
  # @@protoc_insertion_point(class_scope:com.elarian.hera.proto.IndexMapping)
  })
_sym_db.RegisterMessage(IndexMapping)

CustomerIndex = _reflection.GeneratedProtocolMessageType('CustomerIndex', (_message.Message,), {
  'DESCRIPTOR' : _CUSTOMERINDEX,
  '__module__' : 'common_model_pb2'
  # @@protoc_insertion_point(class_scope:com.elarian.hera.proto.CustomerIndex)
  })
_sym_db.RegisterMessage(CustomerIndex)

AppConnectionProps = _reflection.GeneratedProtocolMessageType('AppConnectionProps', (_message.Message,), {
  'DESCRIPTOR' : _APPCONNECTIONPROPS,
  '__module__' : 'common_model_pb2'
  # @@protoc_insertion_point(class_scope:com.elarian.hera.proto.AppConnectionProps)
  })
_sym_db.RegisterMessage(AppConnectionProps)

CustomerNumber = _reflection.GeneratedProtocolMessageType('CustomerNumber', (_message.Message,), {
  'DESCRIPTOR' : _CUSTOMERNUMBER,
  '__module__' : 'common_model_pb2'
  # @@protoc_insertion_point(class_scope:com.elarian.hera.proto.CustomerNumber)
  })
_sym_db.RegisterMessage(CustomerNumber)


# @@protoc_insertion_point(module_scope)



from enum import Enum

MIME_TYPES_BY_ID = {}
MIME_TYPES_BY_NAME = {}


class WellKnowMimeType:

    def __init__(self, name, identifier):
        self.identifier = identifier
        self.name = name
        MIME_TYPES_BY_ID[identifier] = self
        MIME_TYPES_BY_NAME[name] = self


class WellKnowMimeTypes(Enum):
    APPLICATION_AVRO = WellKnowMimeType("application/avro", 0x00)
    APPLICATION_CBOR = WellKnowMimeType("application/cbor", 0x01)
    APPLICATION_GRAPHQL = WellKnowMimeType("application/graphql", 0x02)
    APPLICATION_GZIP = WellKnowMimeType("application/gzip", 0x03)
    APPLICATION_JAVASCRIPT = WellKnowMimeType("application/javascript", 0x04)
    APPLICATION_JSON = WellKnowMimeType("application/json", 0x05)
    APPLICATION_OCTET_STREAM = WellKnowMimeType("application/octet-stream", 0x06)
    APPLICATION_PDF = WellKnowMimeType("application/pdf", 0x07)
    APPLICATION_THRIFT = WellKnowMimeType("application/vnd.apache.thrift.binary", 0x08)
    APPLICATION_PROTOBUF = WellKnowMimeType("application/vnd.google.protobuf", 0x09)
    APPLICATION_XML = WellKnowMimeType("application/xml", 0x0A)
    APPLICATION_ZIP = WellKnowMimeType("application/zip", 0x0B)
    AUDIO_AAC = WellKnowMimeType("audio/aac", 0x0C)
    AUDIO_MP3 = WellKnowMimeType("audio/mp3", 0x0D)
    AUDIO_MP4 = WellKnowMimeType("audio/mp4", 0x0E)
    AUDIO_MPEG3 = WellKnowMimeType("audio/mpeg3", 0x0F)
    AUDIO_MPEG = WellKnowMimeType("audio/mpeg", 0x10)
    AUDIO_OGG = WellKnowMimeType("audio/ogg", 0x11)
    AUDIO_OPUS = WellKnowMimeType("audio/opus", 0x12)
    AUDIO_VORBIS = WellKnowMimeType("audio/vorbis", 0x13)
    IMAGE_BMP = WellKnowMimeType("image/bmp", 0x14)
    IMAGE_GIF = WellKnowMimeType("image/gif", 0x15)
    IMAGE_HEIC_SEQUENCE = WellKnowMimeType("image/heic-sequence", 0x16)
    IMAGE_HEIC = WellKnowMimeType("image/heic", 0x17)
    IMAGE_HEIF_SEQUENCE = WellKnowMimeType("image/heif-sequence", 0x18)
    IMAGE_HEIF = WellKnowMimeType("image/heif", 0x19)
    IMAGE_JPEG = WellKnowMimeType("image/jpeg", 0x1A)
    IMAGE_PNG = WellKnowMimeType("image/png", 0x1B)
    IMAGE_TIFF = WellKnowMimeType("image/tiff", 0x1C)
    MULTIPART_MIXED = WellKnowMimeType("multipart/mixed", 0x1D)
    TEXT_CSS = WellKnowMimeType("text/css", 0x1E)
    TEXT_CSV = WellKnowMimeType("text/csv", 0x1F)
    TEXT_HTML = WellKnowMimeType("text/html", 0x20)
    TEXT_PLAIN = WellKnowMimeType("text/plain", 0x21)
    TEXT_XML = WellKnowMimeType("text/xml", 0x22)
    VIDEO_H264 = WellKnowMimeType("video/H264", 0x23)
    VIDEO_H265 = WellKnowMimeType("video/H265", 0x24)
    VIDEO_VP8 = WellKnowMimeType("video/VP8", 0x25)
    APPLICATION_HESSIAN = WellKnowMimeType("application/x-hessian", 0x26)
    APPLICATION_JAVA_OBJECT = WellKnowMimeType("application/x-java-object", 0x27)
    APPLICATION_CLOUDEVENTS_JSON = WellKnowMimeType("application/cloudevents+json", 0x27)
    # reserved for future use ..
    MESSAGE_RSOCKET_MIMETYPE = WellKnowMimeType("message/x.rsocket.mime-type.v0", 0x7A)
    MESSAGE_RSOCKET_ACCEPT_MIMETYPES = WellKnowMimeType("message/x.rsocket.accept-mime-types.v0", 0x7B)
    MESSAGE_RSOCKET_AUTHENTICATION = WellKnowMimeType("message/x.rsocket.authentication.v0", 0x7C)
    MESSAGE_RSOCKET_TRACING_ZIPKIN = WellKnowMimeType("message/x.rsocket.tracing-zipkin.v0", 0x7D)
    MESSAGE_RSOCKET_ROUTING = WellKnowMimeType("message/x.rsocket.routing.v0", 0x7E)
    MESSAGE_RSOCKET_COMPOSITE_METADATA = WellKnowMimeType("message/x.rsocket.composite-metadata.v0", 0x7F)

    @classmethod
    def from_identifier(cls, identifier):
        return MIME_TYPES_BY_ID[identifier]

    @classmethod
    def from_mime_type(cls, name):
        return MIME_TYPES_BY_NAME[name]

    @classmethod
    def add_wellknown(cls, name, identifier):
        WellKnowMimeType(name, identifier)



import asyncio
from abc import ABCMeta, abstractmethod
from rsocket.timer import IntervalTimer


from rsocket.connection import Connection
from rsocket.frame import CancelFrame, ErrorFrame, KeepAliveFrame, \
    LeaseFrame, MetadataPushFrame, RequestChannelFrame, \
    RequestFireAndForgetFrame, RequestNFrame, RequestResponseFrame, \
    RequestStreamFrame, PayloadFrame, SetupFrame
from rsocket.frame import ErrorCode
from rsocket.payload import Payload
from reactivestreams.publisher import Publisher, DefaultPublisher
from rsocket.handlers import RequestResponseRequester,\
    RequestResponseResponder, RequestStreamRequester, RequestStreamResponder

MAX_STREAM_ID = 0x7FFFFFFF


class RequestHandler(metaclass=ABCMeta):
    """
    An ABC for request handlers.
    """
    def __init__(self, socket):
        super().__init__()
        self.socket = socket

    @abstractmethod
    def request_channel(self, payload: Payload, publisher: Publisher) \
            -> Publisher:
        """
        Bi-directional communication.  A publisher on each end is connected
        to a subscriber on the other end.
        """
        pass

    @abstractmethod
    def request_fire_and_forget(self, payload: Payload):
        pass

    @abstractmethod
    def request_response(self, payload: Payload) -> asyncio.Future:
        pass

    @abstractmethod
    def request_stream(self, payload: Payload) -> Publisher:
        pass


class BaseRequestHandler(RequestHandler):
    def request_channel(self, payload: Payload, publisher: Publisher):
        self.socket.send_error(RuntimeError("Not implemented"))

    def request_fire_and_forget(self, payload: Payload):
        # The requester isn't listening for errors.  Nothing to do.
        pass

    def request_response(self, payload: Payload):
        future = asyncio.Future()
        future.set_exception(RuntimeError("Not implemented"))
        return future

    def request_stream(self, payload: Payload) -> Publisher:
        return DefaultPublisher()


class RSocket:
    def __init__(self, reader, writer, *,
                 handler_factory=BaseRequestHandler, loop=None, server=True,
                 data_encoding=b'utf-8', metadata_encoding=b'utf-8',
                 setup_payload=None, error_handler=None,
                 keep_alive_milliseconds=30000, max_lifetime_milliseconds=120000):
        self._reader = reader
        self._writer = writer
        self._server = server
        self._handler = handler_factory(self)
        self._error_handler = error_handler

        self._next_stream = 2 if self._server else 1
        self._streams = {}

        self._send_queue = asyncio.Queue()
        if not loop:
            loop = asyncio.get_event_loop()

        if not self._server:
            setup = SetupFrame()
            setup.flags_lease = False
            setup.flags_strict = True

            setup.keep_alive_milliseconds = keep_alive_milliseconds
            setup.max_lifetime_milliseconds = max_lifetime_milliseconds
            # setup frame: data encoding, metadata encoding, setup payload
            setup.data_encoding = data_encoding
            setup.metadata_encoding = metadata_encoding
            if setup_payload:
                setup.data = setup_payload.data
                setup.metadata = setup_payload.metadata

            self.send_frame(setup)
            self.keep_alive_timer = IntervalTimer(setup.keep_alive_milliseconds/1000, self.send_keep_alive)

        self._receiver_task = loop.create_task(self._receiver())
        self._sender_task = loop.create_task(self._sender())
        self._error = ErrorFrame()

    async def send_keep_alive(self):
        keep_alive_frame = KeepAliveFrame()
        keep_alive_frame.flags_respond = True
        self.send_frame_directly(keep_alive_frame)

    def allocate_stream(self):
        stream = self._next_stream
        self._next_stream = (self._next_stream + 2) & MAX_STREAM_ID
        while self._next_stream == 0 or self._next_stream in self._streams:
            self._next_stream = (self._next_stream + 2) & MAX_STREAM_ID
        return stream

    def finish_stream(self, stream):
        self._streams.pop(stream, None)

    def send_frame(self, frame):
        self._send_queue.put_nowait(frame)

    def send_frame_directly(self, frame):
        self._writer.write(frame.serialize())

    async def send_error(self, stream, exception):
        error = ErrorFrame()
        error.stream_id = stream
        error.error_code = ErrorCode.APPLICATION_ERROR
        error.data = str(exception).encode()
        self.send_frame(error)

    async def send_response(self, stream, payload, complete=False):
        response = PayloadFrame()
        response.stream_id = stream
        response.flags_complete = complete
        response.data = payload.data
        response.metadata = payload.metadata
        self.send_frame(response)

    async def _receiver(self):
        try:
            connection = Connection()
            while True:
                data = await self._reader.read(1024)
                if not data:
                    self._writer.close()
                    break
                frames = connection.receive_data(data)
                for frame in frames:
                    stream = frame.stream_id
                    if stream and stream in self._streams:
                        self._streams[stream].frame_received(frame)
                        continue
                    if isinstance(frame, CancelFrame):
                        pass
                    elif isinstance(frame, ErrorFrame):
                        if self._error_handler is not None:
                            self._error_handler(Payload(frame.data, frame.metadata))
                            continue
                    elif isinstance(frame, KeepAliveFrame):
                        if frame.flags_respond:
                            frame.flags_respond = False
                            self.send_frame_directly(frame)
                    elif isinstance(frame, LeaseFrame):
                        pass
                    elif isinstance(frame, MetadataPushFrame):
                        pass
                    elif isinstance(frame, RequestChannelFrame):
                        pass
                    elif isinstance(frame, RequestFireAndForgetFrame):
                        asyncio.get_event_loop().run_in_executor(None, self._handler.request_fire_and_forget, Payload(frame.data, frame.metadata))
                    elif isinstance(frame, RequestResponseFrame):
                        stream = frame.stream_id
                        response_handler = self._handler.request_response
                        if asyncio.iscoroutinefunction(response_handler):
                            future_result = await response_handler(Payload(frame.data, frame.metadata))
                        else:
                            future_result = response_handler(Payload(frame.data, frame.metadata))
                        self._streams[stream] = RequestResponseResponder(stream, self, future_result)
                    elif isinstance(frame, RequestStreamFrame):
                        stream = frame.stream_id
                        self._streams[stream] = RequestStreamResponder(
                            stream, self, self._handler.request_stream(
                                Payload(frame.data, frame.metadata)))
                    elif isinstance(frame, RequestNFrame):
                        pass
                    elif isinstance(frame, PayloadFrame):
                        pass
                    elif isinstance(frame, SetupFrame):
                        if frame.flags_lease:
                            lease = LeaseFrame()
                            lease.time_to_live = 10000
                            lease.number_of_requests = 100
                            self.send_frame(lease)
        except asyncio.CancelledError:
            pass

    async def _sender(self):
        try:
            while True:
                frame = await self._send_queue.get()
                self._writer.write(frame.serialize())
                if self._send_queue.empty():
                    await self._writer.drain()
        except asyncio.CancelledError:
            pass

    def fire_and_forget(self, payload):
        frame = RequestFireAndForgetFrame()
        frame.stream_id = self.allocate_stream()
        frame.metadata = payload.metadata
        frame.data = payload.data
        self.send_frame(frame)

    def request_response(self, payload):
        stream = self.allocate_stream()
        requester = RequestResponseRequester(stream, self, payload)
        self._streams[stream] = requester
        return requester

    def request_stream(self, payload: Payload) -> Publisher:
        stream = self.allocate_stream()
        requester = RequestStreamRequester(stream, self, payload)
        self._streams[stream] = requester
        return requester

    # def request_channel(self, local: Publisher) -> Publisher:
    #     stream = self.allocate_stream()
    #     requester = RequestChannelRequester(stream, self, publisher)
    #     self._streams[stream] = requester
    #     return requester

    async def close(self):
        self._sender_task.cancel()
        self._receiver_task.cancel()
        self.keep_alive_timer.cancel()
        await self._sender_task
        await self._receiver_task



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: payment_state.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import common_model_pb2 as common__model__pb2
from . import payment_model_pb2 as payment__model__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='payment_state.proto',
  package='com.elarian.hera.proto',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x13payment_state.proto\x12\x16\x63om.elarian.hera.proto\x1a\x12\x63ommon_model.proto\x1a\x13payment_model.proto\"\xc2\x03\n\x0cPaymentState\x12@\n\x10\x63ustomer_numbers\x18\x01 \x03(\x0b\x32&.com.elarian.hera.proto.CustomerNumber\x12\x45\n\x0f\x63hannel_numbers\x18\x02 \x03(\x0b\x32,.com.elarian.hera.proto.PaymentChannelNumber\x12\x43\n\x0ftransaction_log\x18\x03 \x03(\x0b\x32*.com.elarian.hera.proto.PaymentTransaction\x12H\n\x14pending_transactions\x18\x04 \x03(\x0b\x32*.com.elarian.hera.proto.PaymentTransaction\x12\x42\n\x07wallets\x18\x05 \x03(\x0b\x32\x31.com.elarian.hera.proto.PaymentState.WalletsEntry\x1aV\n\x0cWalletsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x35\n\x05value\x18\x02 \x01(\x0b\x32&.com.elarian.hera.proto.PaymentBalance:\x02\x38\x01\x62\x06proto3'
  ,
  dependencies=[common__model__pb2.DESCRIPTOR,payment__model__pb2.DESCRIPTOR,])




_PAYMENTSTATE_WALLETSENTRY = _descriptor.Descriptor(
  name='WalletsEntry',
  full_name='com.elarian.hera.proto.PaymentState.WalletsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='com.elarian.hera.proto.PaymentState.WalletsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='com.elarian.hera.proto.PaymentState.WalletsEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=453,
  serialized_end=539,
)

_PAYMENTSTATE = _descriptor.Descriptor(
  name='PaymentState',
  full_name='com.elarian.hera.proto.PaymentState',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='customer_numbers', full_name='com.elarian.hera.proto.PaymentState.customer_numbers', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='channel_numbers', full_name='com.elarian.hera.proto.PaymentState.channel_numbers', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='transaction_log', full_name='com.elarian.hera.proto.PaymentState.transaction_log', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='pending_transactions', full_name='com.elarian.hera.proto.PaymentState.pending_transactions', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='wallets', full_name='com.elarian.hera.proto.PaymentState.wallets', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_PAYMENTSTATE_WALLETSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=89,
  serialized_end=539,
)

_PAYMENTSTATE_WALLETSENTRY.fields_by_name['value'].message_type = payment__model__pb2._PAYMENTBALANCE
_PAYMENTSTATE_WALLETSENTRY.containing_type = _PAYMENTSTATE
_PAYMENTSTATE.fields_by_name['customer_numbers'].message_type = common__model__pb2._CUSTOMERNUMBER
_PAYMENTSTATE.fields_by_name['channel_numbers'].message_type = payment__model__pb2._PAYMENTCHANNELNUMBER
_PAYMENTSTATE.fields_by_name['transaction_log'].message_type = payment__model__pb2._PAYMENTTRANSACTION
_PAYMENTSTATE.fields_by_name['pending_transactions'].message_type = payment__model__pb2._PAYMENTTRANSACTION
_PAYMENTSTATE.fields_by_name['wallets'].message_type = _PAYMENTSTATE_WALLETSENTRY
DESCRIPTOR.message_types_by_name['PaymentState'] = _PAYMENTSTATE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

PaymentState = _reflection.GeneratedProtocolMessageType('PaymentState', (_message.Message,), {

  'WalletsEntry' : _reflection.GeneratedProtocolMessageType('WalletsEntry', (_message.Message,), {
    'DESCRIPTOR' : _PAYMENTSTATE_WALLETSENTRY,
    '__module__' : 'payment_state_pb2'
    # @@protoc_insertion_point(class_scope:com.elarian.hera.proto.PaymentState.WalletsEntry)
    })
  ,
  'DESCRIPTOR' : _PAYMENTSTATE,
  '__module__' : 'payment_state_pb2'
  # @@protoc_insertion_point(class_scope:com.elarian.hera.proto.PaymentState)
  })
_sym_db.RegisterMessage(PaymentState)
_sym_db.RegisterMessage(PaymentState.WalletsEntry)


_PAYMENTSTATE_WALLETSENTRY._options = None
# @@protoc_insertion_point(module_scope)



from abc import abstractmethod, ABCMeta
from asyncio import Future, ensure_future

from reactivestreams import Publisher, Subscriber, Subscription
from rsocket.frame import CancelFrame, ErrorFrame, RequestNFrame, \
    RequestResponseFrame, RequestStreamFrame, PayloadFrame
from rsocket.payload import Payload
from rsocket.timer import Timer


class StreamHandler(metaclass=ABCMeta):
    def __init__(self, stream: int, socket):
        super().__init__()
        self.stream = stream
        self.socket = socket

    # Not being marked abstract, since most handlers won't override.
    def frame_sent(self, frame):
        pass

    @abstractmethod
    def frame_received(self, frame):
        pass

    def send_cancel(self):
        """Convenience method for use by requester subclasses."""
        frame = CancelFrame()
        frame.stream_id = self.stream
        self.socket.send_frame(frame)
        self.socket.finish_stream(self.stream)

    def send_request_n(self, n):
        """Convenience method for use by requester subclasses."""
        frame = RequestNFrame()
        frame.stream_id = self.stream
        frame.request_n = n
        self.socket.send_frame(frame)


class RequestResponseRequester(StreamHandler, Future):
    def __init__(self, stream: int, socket, payload: Payload):
        super().__init__(stream, socket)
        request = RequestResponseFrame()
        request.stream_id = self.stream
        request.data = payload.data
        request.metadata = payload.metadata
        self.socket.send_frame(request)

    def frame_received(self, frame):
        if isinstance(frame, PayloadFrame):
            self.set_result(Payload(frame.data, frame.metadata))
            self.socket.finish_stream(self.stream)
        elif isinstance(frame, ErrorFrame):
            self.set_exception(RuntimeError(frame.data))
            self.socket.finish_stream(self.stream)

    def cancel(self, *args, **kwargs):
        super().cancel(*args, **kwargs)
        self.send_cancel()


class RequestResponseResponder(StreamHandler):
    def __init__(self, stream: int, socket, future: Future):
        super().__init__(stream, socket)
        self.future = future
        self.future.add_done_callback(self.future_done)
        if hasattr(self.future, 'timeout'):
            Timer(self.future.timeout, self.timeout(), callback_args=(self,))

    def future_done(self, future):
        if self.future.cancelled():
            pass
        elif not future.exception():
            ensure_future(self.socket.send_response(
                self.stream, future.result(), complete=True))
        else:
            ensure_future(self.socket.send_error(
                self.stream, future.exception()))
        self.socket.finish_stream(self.stream)

    def frame_received(self, frame):
        if isinstance(frame, CancelFrame):
            self.future.cancel()

    def timeout(self):
        if not self.future.done():
            self.future.cancel()


class RequestStreamRequester(StreamHandler, Publisher, Subscription):
    def __init__(self, stream: int, socket, payload: Payload):
        super().__init__(stream, socket)
        self.payload = payload

    def subscribe(self, subscriber):
        # noinspection PyAttributeOutsideInit
        self.subscriber = subscriber

        request = RequestStreamFrame()
        request.initial_request_n = 1
        request.stream_id = self.stream
        request.data = self.payload.data
        request.metadata = self.payload.metadata
        self.socket.send_frame(request)

        self.subscriber.on_subscribe(self)

    def cancel(self, *args, **kwargs):
        super().cancel(*args, **kwargs)
        self.send_cancel()

    def request(self, n):
        self.send_request_n(n)

    def frame_received(self, frame):
        if isinstance(frame, PayloadFrame):
            if frame.data or not frame.flags_complete:
                self.subscriber.on_next(Payload(frame.data, frame.metadata))
            if frame.flags_complete:
                self.subscriber.on_complete()
                self.socket.finish_stream(self.stream)
        elif isinstance(frame, ErrorFrame):
            self.subscriber.on_error(RuntimeError(frame.data))
            self.socket.finish_stream(self.stream)


class RequestStreamResponder(StreamHandler):
    class StreamSubscriber(Subscriber):
        def __init__(self, stream: int, socket):
            super().__init__()
            self.stream = stream
            self.socket = socket

        def on_next(self, value):
            ensure_future(self.socket.send_response(
                self.stream, value, complete=False))

        def on_complete(self):
            ensure_future(self.socket.send_response(
                self.stream, Payload(b'', b''), complete=True))
            self.socket.finish_stream(self.stream)

        def on_error(self, exception):
            ensure_future(self.socket.send_error(self.stream, exception))
            self.socket.finish_stream(self.stream)

        def on_subscribe(self, subscription):
            # noinspection PyAttributeOutsideInit
            self.subscription = subscription

    def __init__(self, stream: int, socket, publisher: Publisher):
        super().__init__(stream, socket)
        self.publisher = publisher
        self.subscriber = self.StreamSubscriber(stream, socket)
        self.publisher.subscribe(self.subscriber)

    def frame_received(self, frame):
        if isinstance(frame, CancelFrame):
            self.subscriber.subscription.cancel()
        elif isinstance(frame, RequestNFrame):
            self.subscriber.subscription.request(frame.request_n)


class RequestChannelRequester(StreamHandler, Publisher, Subscription):
    def __init__(self, stream: int, socket, publisher: Publisher):
        super().__init__(stream, socket)

    def frame_received(self, frame):
        pass

    def subscribe(self, subscriber):
        pass

    def request(self, n):
        pass

    def cancel(self):
        pass



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: app_model.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='app_model.proto',
  package='com.elarian.hera.proto',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0f\x61pp_model.proto\x12\x16\x63om.elarian.hera.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\xaa\x01\n\x10\x43ustomerReminder\x12\x0b\n\x03key\x18\x01 \x01(\t\x12-\n\tremind_at\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12+\n\x08interval\x18\x03 \x01(\x0b\x32\x19.google.protobuf.Duration\x12-\n\x07payload\x18\x04 \x01(\x0b\x32\x1c.google.protobuf.StringValueb\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_wrappers__pb2.DESCRIPTOR,google_dot_protobuf_dot_duration__pb2.DESCRIPTOR,google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,])




_CUSTOMERREMINDER = _descriptor.Descriptor(
  name='CustomerReminder',
  full_name='com.elarian.hera.proto.CustomerReminder',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='com.elarian.hera.proto.CustomerReminder.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='remind_at', full_name='com.elarian.hera.proto.CustomerReminder.remind_at', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='interval', full_name='com.elarian.hera.proto.CustomerReminder.interval', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='payload', full_name='com.elarian.hera.proto.CustomerReminder.payload', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=141,
  serialized_end=311,
)

_CUSTOMERREMINDER.fields_by_name['remind_at'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_CUSTOMERREMINDER.fields_by_name['interval'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_CUSTOMERREMINDER.fields_by_name['payload'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
DESCRIPTOR.message_types_by_name['CustomerReminder'] = _CUSTOMERREMINDER
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

CustomerReminder = _reflection.GeneratedProtocolMessageType('CustomerReminder', (_message.Message,), {
  'DESCRIPTOR' : _CUSTOMERREMINDER,
  '__module__' : 'app_model_pb2'
  # @@protoc_insertion_point(class_scope:com.elarian.hera.proto.CustomerReminder)
  })
_sym_db.RegisterMessage(CustomerReminder)


# @@protoc_insertion_point(module_scope)
